/**
 * @file ex1.c
 * @author Florian Burgener
 * @brief Exercice 1
 * @version 1.0
 * @date 2022-05-03
 *
 * @copyright Copyright (c) 2021
 *
 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#define MATRIX_HEIGHT 5
#define MATRIX_WIDTH 4
#define OPERATION_REPETITION 10

/**
 * @brief Initiliases the matrix.
 *
 * @param m
 * @param n
 * @return double**
 */
double **matrix_init(int m, int n) {
    double **matrix = (double **)malloc(sizeof(double *) * m);

    for (int i = 0; i < m; i += 1) {
        matrix[i] = (double *)malloc(sizeof(double) * n);

        for (int j = 0; j < n; j += 1) {
            matrix[i][j] = sqrt(i + j);
        }
    }

    return matrix;
}

/**
 * @brief Frees the memory of the matrix.
 *
 * @param matrix
 * @param m
 */
void matrix_destroy(double **matrix, int m) {
    for (int i = 0; i < m; i += 1) {
        free(matrix[i]);
    }

    free(matrix);
}

/**
 * @brief Displays the matrix.
 *
 * @param matrix
 * @param m
 * @param n
 */
void matrix_print(double **matrix, int m, int n) {
    for (int i = 0; i < m; i += 1) {
        for (int j = 0; j < n; j += 1) {
            printf("%lf ", matrix[i][j]);
        }

        printf("\n");
    }
}

/**
 * @brief Performs the averaging operation.
 *
 * @param matrix
 * @param m
 * @param n
 * @return double**
 */
double **matrix_averaging(double **matrix, int m, int n) {
    double **output = matrix_init(m, n);

    for (int i = 0; i < m; i += 1) {
        for (int j = 0; j < n; j += 1) {
            double value = 0;

            if (i >= 1 && i <= m - 2 && j >= 1 && j <= n - 2) {
                value += matrix[i][j];
                value += matrix[i + 1][j];
                value += matrix[i - 1][j];
                value += matrix[i][j + 1];
                value += matrix[i][j - 1];
                value /= 5;
            } else {
                value = matrix[i][j];
            }

            output[i][j] = value;
        }
    }

    return output;
}

int main() {
    int m = MATRIX_HEIGHT;
    int n = MATRIX_WIDTH;
    double **matrix = matrix_init(m, n);

    for (int i = 0; i < OPERATION_REPETITION; i += 1) {
        double **matrix_2 = matrix_averaging(matrix, m, n);
        matrix_destroy(matrix, m);
        matrix = matrix_2;
    }

    matrix_print(matrix, m, n);
    matrix_destroy(matrix, m);
    return EXIT_SUCCESS;
}
